
# q-expansion of newform 6014.2.a.c, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6014
weight = 2
poly_data = [-2, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1, 0], [0, 2], [2, 1], [-2, 1], [4, 0], [0, 0], [0, -1], [-6, 0], [0, 4], [4, -4], [-1, 0], [-4, -4], [6, -2], [0, 6], [4, 0], [8, 2], [10, 2], [-4, -6], [2, 6], [2, -3], [2, 4], [-2, 2], [-4, 9], [-2, 6], [1, 0], [-2, 2], [0, -2], [2, -2], [-14, 2], [8, 4], [-8, 2], [10, -8], [-8, -1], [-12, 3], [2, 1], [0, -4], [2, 11], [4, -8], [10, -6], [10, 1], [-4, -1], [4, -10], [0, 6], [-6, 0], [0, -10], [-8, 0], [2, -4], [4, -2], [-4, 0], [0, 10], [-2, 10], [-4, 6], [-22, 4], [8, 7], [-6, -6], [16, 10], [10, -8], [-8, -6], [8, 4], [14, 6], [-2, -16], [-26, -2], [-2, -16], [-10, -13], [14, -2], [2, -9], [-4, -7], [-20, -1], [-20, -9], [2, -9], [18, -8], [-14, -11], [-8, 10], [6, -13], [-6, -12], [0, -12], [10, -12], [30, -2], [24, 7], [-4, 13], [-18, -4], [-18, 2], [4, 14], [8, -15], [6, -13], [-10, -8], [-6, -2], [24, -7], [26, 4], [-22, -10], [-14, -12], [36, -2], [-6, -18], [-4, -8], [4, -7], [-12, 6], [-4, -22], [18, -8], [-20, -7], [10, 11], [8, -4], [10, 4], [30, -6], [16, 13], [-36, 6], [6, 8], [-12, -13], [-6, 0], [6, 13], [-20, -7], [-20, 4], [26, 4], [36, 4], [8, 25], [34, -6], [4, -3], [4, 10], [8, -8], [6, 1], [6, 24], [-2, 26], [6, -14], [-30, -8], [-18, -12], [8, 20], [2, 6], [-12, -2], [44, 4], [8, -20], [-22, 18], [12, 1], [-8, -16], [4, 12], [-4, 10], [32, 3], [34, -8], [-12, 14], [-36, 6], [-20, -12], [38, -6], [32, -16], [0, -4], [-2, -18], [16, 1], [18, -8], [14, 3], [26, -7], [8, 4], [16, 7], [-20, -14], [2, -24], [34, -12], [20, 25], [-2, -11], [46, -4], [-24, 22], [-30, -9], [0, -17], [4, -20], [-8, -16], [-20, 5], [-16, 9], [18, 10], [-16, 0], [-6, 26], [-16, 0], [4, 30], [14, -16], [32, -1], [-36, -2], [-4, 16], [8, 26], [-8, -18], [30, 10], [-6, 23], [26, -24], [30, -20], [18, 6], [8, 28], [-28, -14], [-26, -3], [52, -4], [-10, 5], [12, -1], [-32, -16], [-44, -10], [-26, 18], [-10, 6], [-14, -22], [14, 1], [2, -4], [18, 12], [-8, -17], [-8, 12], [14, 10], [16, 9], [-52, 7], [6, -24], [-20, 8], [10, 5], [26, -14], [-4, -34], [-10, 18], [52, 4], [-4, 24], [-28, 18], [16, 0], [16, -16], [18, 2], [38, 16], [6, -14], [4, -14], [-2, 31], [-2, 24], [4, -22], [38, -10], [-8, 22], [4, 16], [-46, -6], [-2, -13], [-36, 18], [-2, -29], [50, 0], [-44, -14], [-22, 26], [24, 8], [22, -24], [-48, -12], [32, -12], [2, -24], [6, -12], [-18, -8], [-24, -28], [-28, -7], [16, -21], [14, -26], [-2, 40], [-54, -5], [20, 2], [60, 0], [-46, 6], [60, -1], [0, -32], [-14, 40], [-2, 2], [18, 15], [-4, -24], [32, 19], [4, 32], [52, -8], [-18, 31], [-6, 0], [24, -34], [-26, -22], [-10, 24], [-42, 14], [-34, -15], [-12, -42], [-6, -30], [14, 40], [-22, 7], [24, 26], [16, -12], [34, 14], [62, 12], [-14, 24], [-10, -3], [20, -32], [-10, 14], [-22, -1], [-6, 3], [56, 0], [0, 33], [4, -48], [4, 8], [-42, -5], [-10, -52], [-42, 16], [12, -23], [-20, -2], [6, -15], [-12, -8], [48, -10], [-54, -8], [36, 0], [40, -20], [0, 1], [-16, 34], [56, 6], [56, 17], [-38, 31], [0, -28], [-18, -16], [18, -24], [8, -46], [2, 12], [-26, 18], [6, 56], [8, 21], [8, -28], [22, 20], [36, -22], [-4, 46], [10, 16], [0, -37], [38, 12], [40, -3], [42, -28], [-24, -40], [-54, -8], [-24, 18], [-4, 6], [12, 40], [32, -1], [28, 10], [-44, -9], [-16, -6], [20, -44], [-46, -18], [-20, -3], [-16, -4], [2, -6], [-6, 15], [40, -28], [-6, 0], [6, -50], [2, -40], [2, 8], [-56, 8], [10, -52], [-30, 39], [38, -10], [-76, 6], [36, 0], [10, 42], [-56, 8], [-40, 12], [26, 10], [-32, -4], [-28, 6], [-2, -24], [58, -6], [-36, -2], [68, 0], [-50, -9], [16, -12], [10, 25], [60, 15], [-8, 26], [6, -39], [68, 4], [-54, -8], [18, 42], [-20, 52], [-74, -2], [54, 12], [-12, 8], [16, -39], [-6, -42], [-72, -16], [-46, 24], [-22, 6], [32, 40], [26, 20], [-18, -26], [6, 30], [-42, 1], [-40, -16], [-42, -12], [-12, 4], [-10, 34], [-2, 15], [2, 56], [-10, 31], [42, 16], [4, 25], [38, -35], [26, -28], [-26, -6], [-42, -22], [-38, 8], [36, 39], [68, 4], [-40, 15], [16, -11], [32, 2], [42, -25], [-12, 36], [-8, 44], [-16, -18], [18, 12], [-12, 4], [10, -58], [38, 20], [42, -17], [44, 8], [-48, 4], [42, 4], [-16, -51], [60, 22], [22, -56], [-42, 36], [74, 11], [52, -24], [80, 2], [-20, 14], [6, 64], [-2, 6], [72, 17], [36, -14], [40, 43], [50, -20], [-56, 7], [-28, -33], [-60, 24], [-18, -20], [-56, -20], [-14, -51], [14, -10], [14, -1], [14, -24], [-8, 2], [12, 0], [34, -20], [-28, -52], [18, 20], [-10, 16], [-20, 7], [8, 0], [48, -20]]
